@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010, 2011, 2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,15 +17,19 @@ from . import utils
 
 try:
     import argparse
-except ImportError, ex:
-    utils.enhance_import_error(ex, 'argparse', 'python-argparse', 'https://code.google.com/p/argparse/')
+except ImportError as ex:  # <no-coverage>
+    utils.enhance_import_error(ex,
+        'argparse',
+        'python-argparse',
+        'https://pypi.python.org/pypi/argparse'
+    )
     raise
 
 from . import djvu_extra as djvu
 from . import version as version_module
 try:
     from . import xmp
-except ImportError, xmp_import_error:
+except ImportError as xmp_import_error:  # <no-coverage>
     xmp = None
 
 def range_int(x, y, typename):
@@ -41,21 +45,33 @@ dpi_type = range_int(djvu.DPI_MIN, djvu.DPI_MAX, 'dpi')
 losslevel_type = range_int(djvu.LOSS_LEVEL_MIN, djvu.LOSS_LEVEL_MAX, 'loss level')
 subsample_type = range_int(djvu.SUBSAMPLE_MIN, djvu.SUBSAMPLE_MAX, 'subsample')
 
-def slice_type(max_slices=99):
+def slice_type(max_slices=djvu.IW44_N_SLICES_MAX):
 
     def slices(value):
+        result = []
         if ',' in value:
-            result = map(int, value.split(','))
+            prev_slc = 0
+            for slc in value.split(','):
+                slc = int(slc)
+                if slc <= prev_slc:
+                    raise ValueError('non-increasing slice value')
+                result += [slc]
+                prev_slc = slc
+            slc = 0
         elif '+' in value:
-            result = []
-            accum = 0
-            for slice in value.split('+'):
-                accum += int(slice)
-                result += accum,
+            slc = 0
+            for slcinc in value.split('+'):
+                slcinc = int(slcinc)
+                if slcinc <= 0:
+                    raise ValueError('non-increasing slice value')
+                slc += slcinc
+                result += [slc]
         else:
-            result = [int(value)]
-        if not result:
-            raise ValueError('invalid slice specification')
+            slc = int(value)
+            if slc < 0:
+                raise ValueError('invalid slice value')
+            result = [slc]
+        assert len(result) > 0
         if len(result) > max_slices:
             raise ValueError('too many slices')
         return result
@@ -78,12 +94,37 @@ class intact(object):
 def replace_underscores(s):
     return s.replace('_', '-')
 
+def _get_method_params_help(methods):
+    result = ['binarization methods and their parameters:']
+    for name, method in sorted(methods.iteritems()):
+        result += ['  ' + name]
+        for arg in method.args.itervalues():
+            arg_help = arg.name
+            if arg.type in (int, float):
+                arg_help += '=' + 'NX'[arg.type is float]
+                arg_help_paren = []
+                if (arg.min is None) != (arg.max is None):
+                    raise NotImplementedError
+                if arg.min is not None:
+                    arg_help_paren += ['{0} .. {1}'.format(arg.min, arg.max)]
+                if arg.default is not None:
+                    arg_help_paren += ['default: {0}'.format(arg.default)]
+                if arg_help_paren:
+                    arg_help += ' ({0})'.format(', '.join(arg_help_paren))
+            elif arg.type is bool:
+                if arg.default is not False:
+                    raise NotImplementedError
+            else:
+                raise NotImplementedError
+            result += ['  - ' + arg_help]
+    return '\n'.join(result)
+
 class ArgumentParser(argparse.ArgumentParser):
 
     class defaults:
         pageid_template = '{base-ext}.djvu'
         pages_per_dict = 1
-        dpi = djvu.DPI_DEFAULT
+        dpi = None
         fg_slices = [100]
         fg_crcb = djvu.CRCB.full
         fg_subsample = 6
@@ -101,23 +142,35 @@ class ArgumentParser(argparse.ArgumentParser):
         epilog = []
         default = self.defaults
         for p in p_separate, p_encode, p_bundle:
-            epilog += ['%s --help' % p.prog]
+            epilog += ['{prog} --help'.format(prog=p.prog)]
             p.add_argument('-o', '--output', metavar='FILE', help='output filename')
             if p is p_bundle:
                 p.add_argument(
                     '--pageid-template', metavar='TEMPLATE', default=default.pageid_template,
-                    help='naming scheme for page identifiers (default: "%s")' % default.pageid_template
+                    help='naming scheme for page identifiers (default: "{template}")'.format(template=default.pageid_template)
                 )
             else:
                 p.add_argument(
                     '--output-template', metavar='TEMPLATE',
-                    help='naming scheme for output file (e.g. "%s")' % default.pageid_template
+                    help='naming scheme for output file (e.g. "{template}")'.format(template=default.pageid_template)
                 )
             p.add_argument('--losslevel', dest='loss_level', type=losslevel_type, help=argparse.SUPPRESS)
-            p.add_argument('--loss-level', dest='loss_level', type=losslevel_type, metavar='N', help='aggressiveness of lossy compression')
-            p.add_argument('--lossless', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_MIN, help='lossless compression (default)')
-            p.add_argument('--clean', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_CLEAN, help='lossy compression: remove flyspecks')
-            p.add_argument('--lossy', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_LOSSY, help='lossy compression: substitute patterns with small variations')
+            p.add_argument(
+                '--loss-level', dest='loss_level', type=losslevel_type, metavar='N',
+                help='aggressiveness of lossy compression'
+            )
+            p.add_argument(
+                '--lossless', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_MIN,
+                help='lossless compression (default)'
+            )
+            p.add_argument(
+                '--clean', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_CLEAN,
+                help='lossy compression: remove flyspecks'
+            )
+            p.add_argument(
+                '--lossy', dest='loss_level', action='store_const', const=djvu.LOSS_LEVEL_LOSSY,
+                help='lossy compression: substitute patterns with small variations'
+            )
             if p is not p_separate:
                 p.add_argument('--masks', nargs='+', metavar='MASK', help='use pre-generated masks')
                 p.add_argument('--mask', action='append', dest='masks', metavar='MASK', help='use a pre-generated mask')
@@ -125,42 +178,52 @@ class ArgumentParser(argparse.ArgumentParser):
                     if layer == 'fg':
                         p.add_argument(
                             '--fg-slices', type=slice_type(1), metavar='N',
-                            help='number of slices for background (default: %s)' % get_slice_repr(default.fg_slices)
+                            help='number of slices for background (default: {slices})'.format(slices=get_slice_repr(default.fg_slices))
                         )
                     else:
                         p.add_argument(
                             '--bg-slices', type=slice_type(), metavar='N+...+N',
-                            help='number of slices in each forgeground chunk (default: %s)' % get_slice_repr(default.bg_slices)
+                            help='number of slices in each foreground chunk (default: {slices})'.format(slices=get_slice_repr(default.bg_slices))
                         )
-                    default_crcb = getattr(default, '%s_crcb' % layer)
+                    default_crcb = getattr(default, '{lr}_crcb'.format(lr=layer))
                     p.add_argument(
-                        '--%s-crcb' % layer, choices=map(str, djvu.CRCB.values),
-                        help='chrominance encoding for %s (default: %s)' % (layer_name, default_crcb)
+                        '--{lr}-crcb'.format(lr=layer), choices=map(str, djvu.CRCB.values),
+                        help='chrominance encoding for {layer} (default: {crcb})'.format(layer=layer_name, crcb=default_crcb)
                     )
-                    default_subsample = getattr(default, '%s_subsample' % layer)
+                    default_subsample = getattr(default, '{lr}_subsample'.format(lr=layer))
                     p.add_argument(
-                        '--%s-subsample' % layer, type=subsample_type, metavar='N',
-                        help='subsample ratio for %s (default: %d)' % (layer_name, default_subsample)
+                        '--{lr}-subsample'.format(lr=layer), type=subsample_type, metavar='N',
+                        help='subsample ratio for {layer} (default: {n})'.format(layer=layer_name, n=default_subsample)
                     )
                 p.add_argument('--fg-bg-defaults', help=argparse.SUPPRESS, action='store_const', const=1)
             if p is not p_separate:
                 p.add_argument(
                     '-d', '--dpi', type=dpi_type, metavar='N',
-                    help='image resolution (default: %d)' % default.dpi
+                    help='image resolution (default: {dpi})'.format(dpi=djvu.DPI_DEFAULT)
                 )
             if p is p_bundle:
                 p.add_argument(
                     '-p', '--pages-per-dict', type=int, metavar='N',
-                    help='how many pages to compress in one pass (default: %d)' % default.pages_per_dict
+                    help='how many pages to compress in one pass (default: {n})'.format(n=default.pages_per_dict)
                 )
             p.add_argument(
-                '-m', '--method', choices=methods, type=replace_underscores, default=default_method,
-                help='binarization method (default: %s)' % default_method
+                '-m', '--method', choices=methods, metavar='METHOD', type=replace_underscores, default=default_method,
+                help='binarization method (default: {method})'.format(method=default_method)
+            )
+            p.add_argument(
+                '-x', '--param', action='append', dest='params', metavar='NAME[=VALUE]',
+                help='binarization method parameter (can be given more than once)'
             )
             if p is p_encode or p is p_bundle:
                 p.add_argument('--xmp', action='store_true', help='create sidecar XMP metadata (experimental!)')
-            p.add_argument('-v', '--verbose', dest='verbosity', action='append_const', const=None, help='more informational messages')
-            p.add_argument('-q', '--quiet', dest='verbosity', action='store_const', const=[], help='no informational messages')
+            p.add_argument(
+                '-v', '--verbose', dest='verbosity', action='append_const', const=None,
+                help='more informational messages'
+            )
+            p.add_argument(
+                '-q', '--quiet', dest='verbosity', action='store_const', const=[],
+                help='no informational messages'
+            )
             p.add_argument('input', metavar='<input-image>', nargs='+')
             p.set_defaults(
                 masks=[],
@@ -168,11 +231,16 @@ class ArgumentParser(argparse.ArgumentParser):
                 loss_level=djvu.LOSS_LEVEL_MIN,
                 pages_per_dict=default.pages_per_dict,
                 dpi=default.dpi,
-                fg_slices=intact(default.fg_slices), fg_crcb=intact(default.fg_crcb), fg_subsample=intact(default.fg_subsample),
-                bg_slices=intact(default.bg_slices), bg_crcb=intact(default.bg_crcb), bg_subsample=intact(default.bg_subsample),
+                fg_slices=intact(default.fg_slices),
+                bg_slices=intact(default.bg_slices),
+                fg_crcb=intact(default.fg_crcb),
+                bg_crcb=intact(default.bg_crcb),
+                fg_subsample=intact(default.fg_subsample),
+                bg_subsample=intact(default.bg_subsample),
                 verbosity=[None],
                 xmp=False,
             )
+            p.epilog = _get_method_params_help(methods)
         self.epilog = 'more help:\n  ' + '\n  '.join(epilog)
         self.__methods = methods
 
@@ -181,18 +249,53 @@ class ArgumentParser(argparse.ArgumentParser):
             self.__subparsers
         except AttributeError:
             self.__subparsers = self.add_subparsers(parser_class=argparse.ArgumentParser)
+        kwargs.setdefault('formatter_class', argparse.RawDescriptionHelpFormatter)
         p = self.__subparsers.add_parser(name, **kwargs)
         p.set_defaults(_action_=name)
         return p
 
+    def _parse_params(self, options):
+        o = options
+        result = {}
+        for param in o.params or ():
+            if '=' not in param:
+                if param.isdigit() and len(o.method.args) == 1:
+                    [pname] = o.method.args
+                    pvalue = param
+                else:
+                    pname = param
+                    pvalue = True
+            else:
+                [pname, pvalue] = param.split('=', 1)
+            pname = replace_underscores(pname)
+            try:
+                arg = o.method.args[pname]
+            except KeyError:
+                self.error('invalid parameter name {0!r}'.format(pname))
+            try:
+                if (pvalue is True) and (arg.type is not bool):
+                    raise ValueError
+                pvalue = arg.type(pvalue)
+            except ValueError:
+                self.error('invalid parameter {0} value: {1!r}'.format(pname, pvalue))
+            if (arg.min is not None) and pvalue < arg.min:
+                self.error('parameter {0} must be >= {1}'.format(pname, arg.min))
+            if (arg.max is not None) and pvalue > arg.max:
+                self.error('parameter {0} must be <= {1}'.format(pname, arg.max))
+            result[arg.name] = pvalue
+        for arg in o.method.args.itervalues():
+            if (not arg.has_default) and (arg.name not in result):
+                self.error('parameter {0} is not set'.format(arg.name))
+        return result
+
     def parse_args(self, actions):
         o = argparse.ArgumentParser.parse_args(self)
         if o.fg_bg_defaults is None:
             for layer in 'fg', 'bg':
                 namespace = argparse.Namespace()
-                setattr(o, '%s_options' % layer, namespace)
+                setattr(o, '{lr}_options'.format(lr=layer), namespace)
                 for facet in 'slices', 'crcb', 'subsample':
-                    attrname = '%s_%s' % (layer, facet)
+                    attrname = '{lr}_{facet}'.format(lr=layer, facet=facet)
                     value = getattr(o, attrname)
                     if isinstance(value, intact):
                         value = value()
@@ -209,6 +312,7 @@ class ArgumentParser(argparse.ArgumentParser):
             o.pages_per_dict = 1
         action = getattr(actions, vars(o).pop('_action_'))
         o.method = self.__methods[o.method]
+        o.params = self._parse_params(o)
         try:
             if not xmp and o.xmp:
                 raise xmp_import_error
@@ -217,7 +321,15 @@ class ArgumentParser(argparse.ArgumentParser):
         return action(o)
 
 def dump_options(o, multipage=False):
-    yield ('method', o.method.didjvu_name)
+    method_name = o.method.name
+    if o.params:
+        method_name += ' '
+        method_name += ' '.join(
+            '{0}={1}'.format(pname, pvalue)
+            for pname, pvalue
+            in sorted(o.params.iteritems())
+        )
+    yield ('method', method_name)
     if multipage:
         yield ('pages-per-dict', o.pages_per_dict)
     yield ('loss-level', o.loss_level)
@@ -232,4 +344,4 @@ def dump_options(o, multipage=False):
 
 __all__ = ['ArgumentParser', 'dump_options']
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
