@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010, 2011, 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,17 +11,18 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import itertools
+import logging
 import os
 import re
-import logging
 import sys
 
 from . import cli
 from . import djvu_extra as djvu
 from . import filetype
+from . import fs
 from . import gamera_extra as gamera
 from . import ipc
 from . import templates
@@ -39,6 +40,7 @@ def setup_logging():
     logger = logging.getLogger('didjvu.main')
     ipc_logger = logging.getLogger('didjvu.ipc')
     logging.NOSY = (logging.INFO + logging.DEBUG) // 2
+    assert logging.INFO > logging.NOSY > logging.DEBUG
     def nosy(msg, *args, **kwargs):
         logger.log(logging.NOSY, msg, *args, **kwargs)
     logger.nosy = nosy
@@ -53,24 +55,19 @@ def setup_logging():
     handler.setFormatter(formatter)
     ipc_logger.addHandler(handler)
 
+def error(message, *args, **kwargs):
+    if args or kwargs:
+        message = message.format(*args, **kwargs)
+    print('didjvu: error: ' + message, file=sys.stderr)
+    sys.exit(1)
+
 def parallel_for(o, f, *iterables):
     for args in zip(*iterables):
         f(o, *args)
 
 def check_tty():
     if sys.stdout.isatty():
-        print >>sys.stderr, 'I won\'t write binary data to a terminal.'
-        sys.exit(1)
-
-def copy_file(input_file, output_file):
-    length = 0
-    while 1:
-        block = input_file.read(0x1000)
-        if not block:
-            break
-        length += len(block)
-        output_file.write(block)
-    return length
+        error('refusing to write binary data to a terminal')
 
 def get_subsampled_dim(image, subsample):
     width = (image.ncols + subsample - 1) // subsample
@@ -119,7 +116,7 @@ def subsample_fg(image, mask, options):
                 r = (r + n // 2) // n
                 g = (g + n // 2) // n
                 b = (b + n // 2) // n
-                subsampled_image_set(pt, gamera.RGBPixel(r, g , b))
+                subsampled_image_set(pt, gamera.RGBPixel(r, g, b))
             else:
                 subsampled_mask_set(pt, 1)
             x0 += ratio
@@ -142,8 +139,18 @@ def make_layer(image, mask, subsampler, options):
         slices=options.slices, crcb=options.crcb
     )
 
-def image_to_djvu(width, height, image, mask, options):
+def image_dpi(image, options):
     dpi = options.dpi
+    if dpi is None:
+        dpi = image.dpi
+    if dpi is None:
+        dpi = djvu.DPI_DEFAULT
+    dpi = max(dpi, djvu.DPI_MIN)
+    dpi = min(dpi, djvu.DPI_MAX)
+    return dpi
+
+def image_to_djvu(width, height, image, mask, options):
+    dpi = image_dpi(image, options)
     loss_level = options.loss_level
     if options.pages_per_dict > 1:
         # XXX This should probably go the other way round: we run minidjvu
@@ -154,18 +161,22 @@ def image_to_djvu(width, height, image, mask, options):
         image = gamera.to_pil_rgb(image)
         return djvu.Multichunk(width, height, dpi, image=image, sjbz=sjbz)
     else:
-        fg_djvu = make_layer(image, mask, subsample_fg, options.fg_options)
-        bg_djvu = make_layer(image, mask, subsample_bg, options.bg_options)
-        fg44, bg44 = map(djvu.djvu_to_iw44, [fg_djvu, bg_djvu])
-        return djvu.Multichunk(width, height, dpi, fg44=fg44, bg44=bg44, sjbz=sjbz)
+        chunks = dict(sjbz=sjbz)
+        if options.fg_options.slices != [0]:
+            fg_djvu = make_layer(image, mask, subsample_fg, options.fg_options)
+            chunks.update(fg44=djvu.djvu_to_iw44(fg_djvu))
+        if options.bg_options.slices != [0]:
+            bg_djvu = make_layer(image, mask, subsample_bg, options.bg_options)
+            chunks.update(bg44=djvu.djvu_to_iw44(bg_djvu))
+        return djvu.Multichunk(width, height, dpi, **chunks)
 
-def generate_mask(filename, image, method):
+def generate_mask(filename, image, method, params):
     '''
     Generate mask using the provided method (if filename is not None);
     or simply load it from file (if filename is not None).
     '''
     if filename is None:
-        return method(image)
+        return method(image, **params)
     else:
         return gamera.load_image(filename)
 
@@ -173,29 +184,27 @@ _pageid_chars = re.compile('^[A-Za-z0-9_+.-]+$').match
 
 def check_pageid_sanity(pageid):
     if not _pageid_chars(pageid):
-        raise ValueError('Pageid must consist only of lowercase ASCII letters, digits, _, +, - and dot.')
+        raise ValueError('pageid must consist only of lowercase ASCII letters, digits, _, +, - and dot.')
     if pageid[:1] in ('.', '+', '-'):
-        raise ValueError('Pageid cannot start with +, - or a dot.')
+        raise ValueError('pageid cannot start with +, - or a dot.')
     if '..' in pageid:
-        raise ValueError('Pageid cannot contain two consecutive dots.')
+        raise ValueError('pageid cannot contain two consecutive dots.')
     assert pageid == os.path.basename(pageid)
     if pageid.endswith('.djvu'):
         return pageid
     else:
-        raise ValueError('Pageid must end with the .djvu extension.')
-
-def replace_ext(filename, ext):
-    return '%s.%s' % (os.path.splitext(filename)[0], ext)
+        raise ValueError('pageid must end with the .djvu extension.')
 
 class namespace():
     pass
 
 class main():
 
-    compression_info_template = \
-        '%(bits_per_pixel).3f bits/pixel; ' \
-        '%(ratio).3f:1, %(percent_saved).2f%% saved, ' \
-        '%(bytes_in)d bytes in, %(bytes_out)d bytes out'
+    compression_info_template = (
+        '{bits_per_pixel:.3f} bits/pixel; '
+        '{ratio:.3f}:1, {percent_saved:.2f}% saved, '
+        '{bytes_in} bytes in, {bytes_out} bytes out'
+    )
 
     def __init__(self):
         setup_logging()
@@ -206,7 +215,10 @@ class main():
         if len(o.masks) == 0:
             o.masks = [None for x in o.input]
         elif len(o.masks) != len(o.input):
-            raise ValueError('%d input images != %d masks' % (len(o.input), len(o.masks)))
+            error('the number of input images ({0}) does not match the number of masks ({1})',
+                len(o.input),
+                len(o.masks)
+            )
         ipc_logger = logging.getLogger('didjvu.ipc')
         assert logger is not None
         log_level = {
@@ -216,26 +228,31 @@ class main():
         }.get(o.verbosity, logging.DEBUG)
         logger.setLevel(log_level)
         ipc_logger.setLevel(log_level)
+        djvu.require_cli()
         gamera.init()
 
     def check_multi_output(self, o):
         self.check_common(o)
+        pageid_memo = {}
         if o.output is None:
             if o.output_template is not None:
-                o.output = [templates.expand(o.output_template, f, n) for n, f in enumerate(o.input)]
+                o.output = [
+                    templates.expand(o.output_template, f, n, pageid_memo)
+                    for n, f in enumerate(o.input)
+                ]
                 o.xmp_output = [s + '.xmp' if o.xmp else None for s in o.output]
             elif len(o.input) == 1:
                 o.output = [sys.stdout]
                 o.xmp_output = [None]
                 check_tty()
             else:
-                raise ValueError("cannot output multiple files to stdout")
+                error('cannot output multiple files to stdout')
         else:
             if len(o.input) == 1:
                 o.xmp_output = [o.output + '.xmp'] if o.xmp else [None]
                 o.output = [o.output]
             else:
-                raise ValueError("cannot output multiple files to a single file")
+                error('cannot output multiple files to a single file')
         assert len(o.masks) == len(o.output) == len(o.input)
         o.output = (
             open(f, 'wb') if isinstance(f, basestring) else f
@@ -264,36 +281,37 @@ class main():
 
     def encode_one(self, o, image_filename, mask_filename, output, xmp_output):
         bytes_in = os.path.getsize(image_filename)
-        logger.info('%s:' % image_filename)
+        logger.info(image_filename + ':')
         ftype = filetype.check(image_filename)
         if ftype.like(filetype.djvu):
             if ftype.like(filetype.djvu_single):
                 logger.info('- copying DjVu as is')
                 with open(image_filename, 'rb') as djvu_file:
-                    copy_file(djvu_file, output)
+                    fs.copy_file(djvu_file, output)
             else:
                 # TODO: Figure out how many pages the multi-page document
                 # consist of. If it's only one, continue.
-                raise NotImplementedError("I don't know what to do with this file")
+                error('multi-page DjVu documents are not supported as input files')
             return
         logger.info('- reading image')
         image = gamera.load_image(image_filename)
         width, height = image.ncols, image.nrows
-        logger.nosy('- image size: %d x %d', width, height)
-        mask = generate_mask(mask_filename, image, o.method)
+        logger.nosy('- image size: {w} x {h}'.format(w=width, h=height))
+        mask = generate_mask(mask_filename, image, o.method, o.params)
         if xmp_output:
             n_connected_components = len(mask.cc_analysis())
         logger.info('- converting to DjVu')
         djvu_doc = image_to_djvu(width, height, image, mask, options=o)
         djvu_file = djvu_doc.save()
         try:
-            bytes_out = copy_file(djvu_file, output)
+            bytes_out = fs.copy_file(djvu_file, output)
         finally:
             djvu_file.close()
         bits_per_pixel = 8.0 * bytes_out / (width * height)
         ratio = 1.0 * bytes_in / bytes_out
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
-        logger.info('- %s', self.compression_info_template % locals())
+        compression_info = self.compression_info_template.format(**locals())
+        logger.info('- ' + compression_info)
         if xmp_output:
             logger.info('- saving XMP metadata')
             metadata = xmp.metadata()
@@ -308,18 +326,18 @@ class main():
             metadata.write(xmp_output)
 
     def separate_one(self, o, image_filename, output):
-        logger.info('%s:', image_filename)
+        logger.info(image_filename + ':')
         ftype = filetype.check(image_filename)
         if ftype.like(filetype.djvu):
             # TODO: Figure out how many pages the document consist of.
             # If it's only one, extract the existing mask.
-            raise NotImplementedError("I don't know what to do with this file")
+            error('DjVu documents are not supported as input files')
         logger.info('- reading image')
         image = gamera.load_image(image_filename)
         width, height = image.ncols, image.nrows
-        logger.nosy('- image size: %d x %d' % (width, height))
+        logger.nosy('- image size: {w} x {h}'.format(w=width, h=height))
         logger.info('- thresholding')
-        mask = generate_mask(None, image, o.method)
+        mask = generate_mask(None, image, o.method, o.params)
         logger.info('- saving')
         if output is not sys.stdout:
             # A real file
@@ -328,7 +346,7 @@ class main():
             tmp_output = temporary.file(suffix='.png')
             try:
                 mask.save_PNG(tmp_output.name)
-                copy_file(tmp_output, output)
+                fs.copy_file(tmp_output, output)
             finally:
                 tmp_output.close()
 
@@ -340,9 +358,10 @@ class main():
 
     def bundle(self, o):
         self.check_single_output(o)
-        if o.pages_per_dict <= 1:
+        if (o.pages_per_dict <= 1) or (len(o.input) <= 1):
             self.bundle_simple(o)
         else:
+            ipc.require('minidjvu')
             self.bundle_complex(o)
         [xmp_output] = o.xmp_output
         if xmp_output:
@@ -364,40 +383,42 @@ class main():
         with temporary.directory() as tmpdir:
             bytes_in = 0
             component_filenames = []
+            pageid_memo = {}
             for page, (input, mask) in enumerate(zip(o.input, o.masks)):
                 bytes_in += os.path.getsize(input)
-                pageid = templates.expand(o.pageid_template, input, page)
-                # TODO: Check for filename conflicts.
+                pageid = templates.expand(o.pageid_template, input, page, pageid_memo)
                 check_pageid_sanity(pageid)
                 component_filenames += os.path.join(tmpdir, pageid),
             parallel_for(o, self._bundle_simple_page, o.input, o.masks, component_filenames)
             logger.info('bundling')
             djvu_file = djvu.bundle_djvu(*component_filenames)
             try:
-                bytes_out = copy_file(djvu_file, output)
+                bytes_out = fs.copy_file(djvu_file, output)
             finally:
                 djvu_file.close()
-        bits_per_pixel = float('nan') # FIXME!
+        bits_per_pixel = float('nan')  # FIXME!
         ratio = 1.0 * bytes_in / bytes_out
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
-        logger.nosy(self.compression_info_template % locals())
+        compression_info = self.compression_info_template.format(**locals())
+        logger.nosy(compression_info)
 
     def _bundle_complex_page(self, o, page, minidjvu_in_dir, image_filename, mask_filename, pixels):
-        logger.info('%s:', image_filename)
+        logger.info(image_filename + ':')
         ftype = filetype.check(image_filename)
         if ftype.like(filetype.djvu):
-            # TODO: Allow to merge existing documents (even multi-page ones).
-            raise NotImplementedError("I don't know what to do with this file")
+            # TODO: Allow merging existing documents (even multi-page ones).
+            error('DjVu documents are not supported as input files')
         logger.info('- reading image')
         image = gamera.load_image(image_filename)
+        dpi = image_dpi(image, o)
         width, height = image.ncols, image.nrows
         pixels[0] += width * height
-        logger.nosy('- image size: %d x %d', width, height)
-        mask = generate_mask(mask_filename, image, o.method)
+        logger.nosy('- image size: {w} x {h}'.format(w=width, h=height))
+        mask = generate_mask(mask_filename, image, o.method, o.params)
         logger.info('- converting to DjVu')
         page.djvu = image_to_djvu(width, height, image, mask, options=o)
         image = mask = None
-        page.sjbz = djvu.Multichunk(width, height, o.dpi, sjbz=page.djvu['sjbz'])
+        page.sjbz = djvu.Multichunk(width, height, dpi, sjbz=page.djvu['sjbz'])
         page.sjbz_symlink = os.path.join(minidjvu_in_dir, page.pageid)
         os.symlink(page.sjbz.save().name, page.sjbz_symlink)
 
@@ -407,13 +428,17 @@ class main():
             bytes_in = 0
             pixels = [0]
             page_info = []
+            pageid_memo = {}
             for pageno, (image_filename, mask_filename) in enumerate(zip(o.input, o.masks)):
                 page = namespace()
                 page_info += page,
                 bytes_in += os.path.getsize(image_filename)
-                page.pageid = templates.expand(o.pageid_template, image_filename, pageno)
-                check_pageid_sanity(page.pageid)
-                # TODO: Check for filename conflicts.
+                page.pageid = templates.expand(o.pageid_template, image_filename, pageno, pageid_memo)
+                try:
+                    check_pageid_sanity(page.pageid)
+                except ValueError as exc:
+                    error(str(exc))
+            del page  # quieten pyflakes
             parallel_for(o, self._bundle_complex_page,
                 page_info,
                 itertools.repeat(minidjvu_in_dir),
@@ -431,16 +456,17 @@ class main():
                     '--aggression', str(o.loss_level),
                     '--pages-per-dict', str(o.pages_per_dict),
                 ]
+                assert len(page_info) > 1  # minidjvu won't create single-page indirect documents
                 arguments += [page.sjbz_symlink for page in page_info]
                 index_filename = temporary.name(prefix='__index__.', suffix='.djvu', dir=minidjvu_out_dir)
-                index_filename = os.path.basename(index_filename) # FIXME: Name conflicts are still possible!
+                index_filename = os.path.basename(index_filename)  # FIXME: Name conflicts are still possible!
                 arguments += [index_filename]
                 ipc.Subprocess(arguments, preexec_fn=chdir).wait()
                 os.remove(os.path.join(minidjvu_out_dir, index_filename))
                 component_filenames = []
                 for pageno, page in enumerate(page_info):
                     if pageno % o.pages_per_dict == 0:
-                        iff_name = replace_ext(page_info[pageno].pageid, 'iff')
+                        iff_name = fs.replace_ext(page_info[pageno].pageid, 'iff')
                         iff_name = os.path.join(minidjvu_out_dir, iff_name)
                         component_filenames += [iff_name]
                     sjbz_name = os.path.join(minidjvu_out_dir, page.pageid)
@@ -454,12 +480,13 @@ class main():
                 logger.info('bundling')
                 djvu_file = djvu.bundle_djvu(*component_filenames)
                 try:
-                    bytes_out = copy_file(djvu_file, output)
+                    bytes_out = fs.copy_file(djvu_file, output)
                 finally:
                     djvu_file.close()
         bits_per_pixel = 8.0 * bytes_out / pixels
         ratio = 1.0 * bytes_in / bytes_out
         percent_saved = (1.0 * bytes_in - bytes_out) * 100 / bytes_in
-        logger.nosy('%s', self.compression_info_template % locals())
+        compression_info = self.compression_info_template.format(**locals())
+        logger.nosy(compression_info)
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
