@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,8 +13,6 @@
 
 '''wrappers for the DjVuLibre utilities'''
 
-from __future__ import with_statement
-
 import os
 import re
 import struct
@@ -36,6 +34,8 @@ SUBSAMPLE_DEFAULT = 3
 SUBSAMPLE_MAX = 12
 
 IW44_SLICES_DEFAULT = (74, 89, 99)
+IW44_N_SLICES_MAX = 63
+# http://sourceforge.net/p/djvu/djvulibre-git/ci/release.3.5.27.1/tree/tools/c44.cpp#l246
 
 class Crcb(object):
 
@@ -51,9 +51,6 @@ class Crcb(object):
     def __str__(self):
         return self._name
 
-    def __repr__(self):
-        return '%s.CRCB.%s' % (type(self).__module__, self._name)
-
 class CRCB:
 
     values = [
@@ -71,36 +68,43 @@ def bitonal_to_djvu(image, dpi=300, loss_level=0):
     pbm_file = temporary.file(suffix='.pbm')
     image.save(pbm_file.name)
     djvu_file = temporary.file(suffix='.djvu', mode='r+b')
-    args = ['cjb2', '-losslevel', str(loss_level), pbm_file.name, djvu_file.name]
+    args = [
+        'cjb2',
+        '-dpi', str(dpi),
+        '-losslevel', str(loss_level),
+        pbm_file.name,
+        djvu_file.name
+    ]
     return ipc.Proxy(djvu_file, ipc.Subprocess(args).wait, [pbm_file])
 
 def photo_to_djvu(image, dpi=100, slices=IW44_SLICES_DEFAULT, gamma=2.2, mask_image=None, crcb=CRCB.normal):
     ppm_file = temporary.file(suffix='.ppm')
-    temporaries = [ppm_file]
     image.save(ppm_file.name)
-    djvu_file = temporary.file(suffix='.djvu', mode='r+b')
     if not isinstance(crcb, Crcb):
         raise TypeError
-    args = [
-        'c44',
-        '-dpi', str(dpi),
-        '-slice', ','.join(map(str, slices)),
-        '-gamma', '%.1f' % gamma,
-        '-crcb%s' % crcb,
-    ]
-    if mask_image is not None:
-        pbm_file = temporary.file(suffix='.pbm')
-        mask_image.save(pbm_file.name)
-        args += ['-mask', pbm_file.name]
-        temporaries += [pbm_file]
-    args += [ppm_file.name, djvu_file.name]
-    return ipc.Proxy(djvu_file, ipc.Subprocess(args).wait, temporaries)
+    with temporary.directory() as djvu_dir:
+        args = [
+            'c44',
+            '-dpi', str(dpi),
+            '-slice', ','.join(map(str, slices)),
+            '-gamma', '{0:.1f}'.format(gamma),
+            '-crcb{0}'.format(crcb),
+        ]
+        if mask_image is not None:
+            pbm_file = temporary.file(suffix='.pbm')
+            mask_image.save(pbm_file.name)
+            args += ['-mask', pbm_file.name]
+        djvu_path = os.path.join(djvu_dir, 'result.djvu')
+        args += [ppm_file.name, djvu_path]
+        ipc.Subprocess(args).wait()
+        return temporary.hardlink(djvu_path, suffix='.djvu')
 
 def djvu_to_iw44(djvu_file):
     # TODO: Use Multichunk.
     iw44_file = temporary.file(suffix='.iw44')
-    args = ['djvuextract', djvu_file.name, 'BG44=%s' % iw44_file.name]
-    return ipc.Proxy(iw44_file, ipc.Subprocess(args).wait, [djvu_file])
+    args = ['djvuextract', djvu_file.name, 'BG44=' + iw44_file.name]
+    with open(os.devnull, 'wb') as dev_null:
+        return ipc.Proxy(iw44_file, ipc.Subprocess(args, stderr=dev_null).wait, [djvu_file])
 
 def _int_or_none(x):
     if x is None:
@@ -129,8 +133,8 @@ class Multichunk(object):
         self.height = _int_or_none(height)
         self.dpi = _int_or_none(dpi)
         self._chunks = {}
-        self._dirty = set() # Chunks that need to be re-read from the file.
-        self._pristine = False # Should save() be a no-op?
+        self._dirty = set()  # Chunks that need to be re-read from the file.
+        self._pristine = False  # Should save() be a no-op?
         self._file = None
         for (k, v) in chunks.iteritems():
             self[k] = v
@@ -153,7 +157,7 @@ class Multichunk(object):
                     else:
                         keys.add(key.lower())
                 else:
-                    ValueError
+                    raise ValueError
         finally:
             dump.wait()
         self._chunks = dict((key, None) for key in keys)
@@ -202,10 +206,11 @@ class Multichunk(object):
         args = ['djvuextract', self._file.name]
         chunk_files = {}
         for key in self._dirty:
-            chunk_file = temporary.file(suffix='.%s-chunk' % key)
-            args += ['%s=%s' % (self._chunk_names[key], chunk_file.name)]
+            chunk_file = temporary.file(suffix='.{key}-chunk'.format(key=key))
+            args += ['{key}={path}'.format(key=self._chunk_names[key], path=chunk_file.name)]
             chunk_files[key] = chunk_file
-        djvuextract = ipc.Subprocess(args, stderr=open(os.devnull, 'w'))
+        with open(os.devnull, 'wb') as dev_null:
+            djvuextract = ipc.Subprocess(args, stderr=dev_null)
         for key in self._chunks:
             self._chunks[key] = ipc.Proxy(chunk_files[key], djvuextract.wait, [self._file])
             self._dirty.discard(key)
@@ -224,25 +229,37 @@ class Multichunk(object):
             raise ValueError
         if len(self._chunks) == 0:
             raise ValueError
-        args = ['djvumake', None, 'INFO=%d,%d,%d' % (self.width, self.height, self.dpi)]
+        args = ['djvumake', None, 'INFO={w},{h},{r}'.format(w=self.width, h=self.height, r=self.dpi)]
+        incl_dir = None
         for key, value in sorted(self._chunks.iteritems(), key=_chunk_order):
             try:
                 key = self._chunk_names[key]
             except KeyError:
                 pass
+            if key == 'INCL':
+                # This is tricky. DjVuLibre used to require (at least until v3.5.27) full path for INCL,
+                # but now it requires just basename:
+                # http://sourceforge.net/p/djvu/djvulibre-git/ci/2fea3cdd3eb2b9ae60a43a851dbd838b6939af4b/
+                # Let's work around this inconsistency.
+                new_incl_dir = os.path.dirname(value)
+                if incl_dir is None:
+                    incl_dir = new_incl_dir
+                elif incl_dir != new_incl_dir:
+                    raise ValueError
+                value = os.path.basename(value)
             if not isinstance(value, basestring):
                 value = value.name
             if key == 'BG44':
                 value += ':99'
-            args += ['%s=%s' % (key, value)]
+            args += ['{key}={value}'.format(key=key, value=value)]
+        def chdir():
+            os.chdir(incl_dir or '.')
         with temporary.directory() as tmpdir:
             djvu_filename = args[1] = os.path.join(tmpdir, 'result.djvu')
-            ipc.Subprocess(args).wait()
+            ipc.Subprocess(args, preexec_fn=chdir).wait()
             self._chunks.pop('PPM', None)
             assert 'PPM' not in self._dirty
-            djvu_new_filename = temporary.name(suffix='.djvu')
-            os.link(djvu_filename, djvu_new_filename)
-            self._file = temporary.wrapper(file(djvu_new_filename, mode='r+b'), djvu_new_filename)
+            self._file = temporary.hardlink(djvu_filename)
             self._pristine = True
             return self._file
 
@@ -296,15 +313,27 @@ def bundle_djvu(*component_filenames):
     if any(c.endswith('.iff') for c in component_filenames):
         # We can't use ``djvm -c``.
         return bundle_djvu_via_indirect(*component_filenames)
-        raise NotImplementedError
     else:
         djvu_file = temporary.file(suffix='.djvu')
         args = ['djvm', '-c', djvu_file.name]
         args += component_filenames
         return ipc.Proxy(djvu_file, ipc.Subprocess(args).wait, None)
 
+def require_cli():
+    ipc.require(
+        'cjb2',
+        'c44',
+        'djvuextract',
+        'djvudump',
+        'djvumake',
+        'bzz',
+        'djvmcvt',
+    )
+
 __all__ = [
     'bitonal_to_djvu', 'photo_to_djvu', 'djvu_to_iw44',
+    'bundle_djvu',
+    'require_cli',
     'Multichunk',
     'DPI_MIN', 'DPI_DEFAULT', 'DPI_MAX',
     'LOSS_LEVEL_MIN', 'LOSS_LEVEL_CLEAN', 'LOSS_LEVEL_LOSSY', 'LOSS_LEVEL_MAX',
@@ -313,4 +342,4 @@ __all__ = [
     'CRCB',
 ]
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
