@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2010 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2010-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -11,7 +11,9 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
-from __future__ import with_statement
+'''
+filetype detection
+'''
 
 class generic(object):
 
@@ -38,4 +40,4 @@ def check(filename):
                 cls = djvu_single
     return cls
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
