@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2009, 2010, 2011, 2012, 2013 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2009-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,7 +13,9 @@
 
 '''bridge to the Gamera framework'''
 
+import collections
 import ctypes
+import math
 import re
 import sys
 import warnings
@@ -22,8 +24,12 @@ from . import utils
 
 try:
     from PIL import Image as PIL
-except ImportError, ex:
-    utils.enhance_import_error(ex, 'Python Imaging Library', 'python-imaging', 'http://www.pythonware.com/products/pil/')
+except ImportError as ex:  # <no-coverage>
+    utils.enhance_import_error(ex,
+        'Python Imaging Library',
+        'python-imaging',
+        'http://www.pythonware.com/products/pil/'
+    )
     raise
 else:
     # Gamera (<< 3.4.0) still expects that PIL can be imported as Image
@@ -31,50 +37,126 @@ else:
 
 try:
     import gamera
-except ImportError, ex:
-    utils.enhance_import_error(ex, 'Gamera', 'python-gamera', 'http://gamera.sourceforge.net/')
+except ImportError as ex:  # <no-coverage>
+    utils.enhance_import_error(ex,
+        'Gamera',
+        'python-gamera',
+        'http://gamera.sourceforge.net/'
+    )
     raise
+del gamera  # quieten pyflakes
 
 from gamera import __version__ as version
 from gamera.core import load_image as _load_image
 from gamera.core import init_gamera as _init
 from gamera.core import Image, RGB, GREYSCALE, ONEBIT, Point, Dim, RGBPixel
 from gamera.plugins.pil_io import from_pil as _from_pil
+import gamera.args
 
 def has_version(*req_version):
     return tuple(map(int, version.split('.'))) >= req_version
 
 def load_image(filename):
+    pil_image = PIL.open(filename)
+    [xdpi, ydpi] = pil_image.info.get('dpi', (0, 0))
+    if xdpi <= 1 or ydpi <= 1:
+        # not reliable
+        dpi = None
+    else:
+        dpi = int(round(
+            math.hypot(xdpi, ydpi) /
+            math.hypot(1, 1)
+        ))
     try:
-        # Natively, Gamera supports only TIFF and PNG formats. However, it
-        # supports wider variety of TIFFs than PIL.
-        return _load_image(filename)
+        # Gamera handles only a few TIFF color modes correctly.
+        # https://bugs.debian.org/784374
+        if pil_image.mode not in ['1', 'I;16', 'L', 'RGB']:
+            raise IOError
+        # Gamera supports more TIFF compression formats that PIL.
+        # https://mail.python.org/pipermail/image-sig/2003-July/002354.html
+        image = _load_image(filename)
     except IOError:
-        pil_image = PIL.open(filename)
         # Gamera supports importing only 8-bit and RGB from PIL:
         if pil_image.mode == '1':
             pil_image = pil_image.convert('L')
         elif pil_image.mode not in ('RGB', 'L'):
             pil_image = pil_image.convert('RGB')
         assert pil_image.mode in ('RGB', 'L')
-        return _from_pil(pil_image)
+        image = _from_pil(pil_image)
+    image.dpi = dpi
+    return image
 
-def colorspace_wrapper(plugin):
+class Argument(object):
 
-    pixel_types = plugin.self_type.pixel_types
+    _type_map = {
+        gamera.args.Int: int,
+        gamera.args.Real: float,
+        gamera.args.Check: bool,
+    }
 
-    def new_plugin(image, method=[None]):
+    def __init__(self, arg):
+        self.name = arg.name.replace(' ', '-').replace('_', '-')
+        for gtype, ptype in self._type_map.iteritems():
+            if isinstance(arg, gtype):
+                self.type = ptype
+                break
+        else:
+            raise NotImplementedError(
+                'argument {0}: unsupported type {1!r}' + repr(self.name, arg)
+            )  # <no-coverage>
+        if self.type in (int, float):
+            [self.min, self.max] = arg.rng
+            if self.min == -gamera.args.DEFAULT_MAX_ARG_NUMBER:
+                self.min = None
+            if self.max == gamera.args.DEFAULT_MAX_ARG_NUMBER:
+                self.max = None
+        else:
+            self.min = self.max = None
+        self.has_default = arg.has_default
+        self.default = arg.default
+        if isinstance(self.default, gamera.args.CNoneDefault):
+            self.default = None
+        elif self.has_default:
+            if not isinstance(self.default, self.type):
+                raise TypeError(
+                    'argument {0}: type({1!r}) should be {2}'.format(self.name, self.default, self.type.__name__)
+                )  # <no-coverage>
+        else:
+            self.default = None
+
+class Plugin(object):
+
+    def __init__(self, plugin, name):
+        self._plugin = plugin
+        self._pixel_types = plugin.self_type.pixel_types
+        self._method = None
+        self.name = name
+        if sys.version_info >= (2, 7):
+            self.args = collections.OrderedDict()
+        else:  # <no-coverage>
+            # Oh well.
+            self.args = {}
+        for arg in plugin.args:
+            if arg.name == 'storage format':
+                continue
+            arg = Argument(arg)
+            self.args[arg.name] = arg
+
+    def __call__(self, image, **kwargs):
+        kwargs = dict(
+            (key.replace('-', '_'), value)
+            for key, value
+            in kwargs.iteritems()
+        )
+        pixel_types = self._pixel_types
         if image.data.pixel_type not in pixel_types:
             if RGB in pixel_types:
                 image = image.to_rgb()
             elif GREYSCALE in pixel_types:
                 image = image.to_greyscale()
-        if method[0] is None:
-            method[0] = plugin()
-        return method[0](image)
-
-    new_plugin.__name__ = plugin.__name__
-    return new_plugin
+        if self._method is None:
+            self._method = self._plugin()
+        return self._method(image, **kwargs)
 
 def _load_methods():
     replace_suffix = re.compile('_threshold$').sub
@@ -83,6 +165,7 @@ def _load_methods():
         from gamera.plugins.threshold import bernsen_threshold
         from gamera.plugins.threshold import djvu_threshold
         from gamera.plugins.threshold import otsu_threshold
+        from gamera.plugins.threshold import threshold as global_threshold
         from gamera.plugins.threshold import tsai_moment_preserving_threshold as tsai
         # TODO: from gamera.plugins.binarization import gatos_threshold
         from gamera.plugins.binarization import niblack_threshold
@@ -98,8 +181,7 @@ def _load_methods():
             continue
         name = replace_suffix('', name)
         name = name.replace('_', '-')
-        method = colorspace_wrapper(plugin)
-        method.didjvu_name = name
+        method = Plugin(plugin, name)
         methods[name] = method
     return methods
 
@@ -114,7 +196,7 @@ def to_pil_rgb(image):
 def to_pil_1bpp(image):
     return image.to_greyscale().to_pil()
 
-def _decref(o):
+def _decref(o):  # <no-coverage>
     '''
     Forcibly decrease refcount of the object by 1.
     '''
@@ -124,7 +206,7 @@ def _decref(o):
     memmove.restype = ctypes.py_object
     return memmove(o, None, 0)
 
-def _monkeypatch_to_raw_string():
+def _monkeypatch_to_raw_string():  # <no-coverage>
     '''
     Monkey-patch to _to_raw_string plugin function to return objects with
     correct refcounts.
@@ -146,7 +228,7 @@ def init():
     test_image = Image((0, 0), (5, 5), RGB)
     test_string = test_image._to_raw_string()
     refcount = sys.getrefcount(test_string)
-    if refcount == 3:
+    if refcount == 3:  # <no-coverage>
         # See: https://tech.groups.yahoo.com/group/gamera-devel/message/2068
         warnings.warn(RuntimeWarning('Working around memory leak in the Gamera library'), stacklevel=2)
         _monkeypatch_to_raw_string()
@@ -157,4 +239,22 @@ def init():
         assert refcount == 2
     return result
 
-# vim:ts=4 sw=4 et
+__all__ = [
+    # classes:
+    'Dim',
+    'Image',
+    'Point',
+    'RGBPixel',
+    # pixel types:
+    'ONEBIT',
+    'GREYSCALE',
+    'RGB',
+    # functions:
+    'init',
+    'load_image',
+    'methods',
+    'to_pil_1bpp',
+    'to_pil_rgb',
+]
+
+# vim:ts=4 sts=4 sw=4 et
