@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012-2015 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,11 +22,15 @@ from .. import version
 
 try:
     from . import libxmp_backend as default_backend
-except ImportError, exc:
+except ImportError as exc:  # <no-coverage>
     try:
         from . import pyexiv2_backend as default_backend
     except ImportError:
-        utils.enhance_import_error(exc, 'python-xmp-toolkit', 'python-libxmp', 'https://code.google.com/p/python-xmp-toolkit/')
+        utils.enhance_import_error(exc,
+            'python-xmp-toolkit',
+            'python-libxmp',
+            'https://github.com/python-xmp-toolkit/python-xmp-toolkit'
+        )
         raise exc
 
 def gen_uuid():
@@ -61,16 +65,16 @@ def metadata(backend=default_backend):
 
     class Metadata(backend.MetadataBase):
 
-        def update(self, media_type, internal_properties={}):
+        def update(self, media_type, internal_properties=()):
             instance_id = gen_uuid()
             now = timestamp.now()
             original_media_type = self.get('dc.format')
             # TODO: try to guess original media type
             self['dc.format'] = media_type
             if original_media_type is not None:
-                event_params = 'from %s to %s' % (original_media_type, media_type)
+                event_params = 'from {0} to {1}'.format(original_media_type, media_type)
             else:
-                event_params = 'to %s' % (media_type,)
+                event_params = 'to {0}'.format(media_type)
             self['xmp.ModifyDate'] = now
             self['xmp.MetadataDate'] = now
             self['xmpMM.InstanceID'] = instance_id
@@ -87,7 +91,7 @@ def metadata(backend=default_backend):
         def import_(self, image_filename):
             try:
                 file = open(image_filename + '.xmp', 'rb')
-            except (OSError, IOError), ex:
+            except (OSError, IOError) as ex:
                 if ex.errno == errno.ENOENT:
                     return
                 raise
@@ -103,4 +107,4 @@ def metadata(backend=default_backend):
 
 __all__ = ['metadata']
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
