@@ -1,6 +1,6 @@
 # encoding=UTF-8
 
-# Copyright © 2012 Jakub Wilk <jwilk@jwilk.net>
+# Copyright © 2012-2014 Jakub Wilk <jwilk@jwilk.net>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -35,7 +35,8 @@ xmp_register_namespace('didjvu', ns.didjvu)
 
 try:
     etree.register_namespace
-except AttributeError:
+except AttributeError:  # <no-coverage>
+    # Python 2.6
     def et_register_namespace(prefix, uri):
         import xml.etree.ElementTree as etree
         etree._namespace_map[uri] = prefix
@@ -87,6 +88,9 @@ class datetime_for_pyexiv2(datetime.datetime):
         # Let's fool it into thinking it's always non-zero.
         return self.__second or self.__almost_zero
 
+def nstag(ns, tag):
+    return '{{{ns}}}{tag}'.format(ns=ns, tag=tag)
+
 class MetadataBase(object):
 
     def _reload(self):
@@ -110,15 +114,15 @@ class MetadataBase(object):
         description = None
         try:
             xmp_find = xmp.iterfind
-        except AttributeError:
+        except AttributeError:  # <no-coverage>
             # Python 2.6
             xmp_find = xmp.findall
-        for description in xmp_find('.//{%s}Description' % ns.rdf):
+        for description in xmp_find('.//' + nstag(ns.rdf, 'Description')):
             pass
         if description is None:
             raise XmpError('Cannot add xmpMM:History')
-        e_description = etree.SubElement(description, '{%s}History' % ns.xmpmm)
-        etree.SubElement(e_description, '{%s}Seq' % ns.rdf)
+        e_description = etree.SubElement(description, nstag(ns.xmpmm, 'History'))
+        etree.SubElement(e_description, nstag(ns.rdf, 'Seq'))
         fp.seek(0)
         fp.truncate()
         xmp.write(fp, encoding='UTF-8')
@@ -132,8 +136,10 @@ class MetadataBase(object):
 
     def __init__(self):
         self._fp = fp = temporary.file(suffix='.xmp')
-        fp.write('<x:xmpmeta xmlns:x="adobe:ns:meta/" xmlns:rdf="%s">'
-            '<rdf:RDF/></x:xmpmeta>' % ns.rdf
+        fp.write(
+            '<x:xmpmeta xmlns:x="adobe:ns:meta/" xmlns:rdf="{ns.rdf}">'
+            '<rdf:RDF/>'
+            '</x:xmpmeta>'.format(ns=ns)
         )
         self._reload()
         self._original_meta = self._meta
@@ -173,13 +179,13 @@ class MetadataBase(object):
         for key, value in event.items:
             if value is None:
                 continue
-            self['xmpMM.History[%d]/stEvt:%s' % (index, key)] = value
+            self['xmpMM.History[{i}]/stEvt:{key}'.format(i=index, key=key)] = value
 
     def append_to_history(self, event):
         self._add_history()
         keys = self._meta.xmp_keys
         for i in itertools.count(1):
-            key = 'Xmp.xmpMM.History[%d]' % i
+            key = 'Xmp.xmpMM.History[{i}]'.format(i=i)
             if key not in keys:
                 break
         return self.add_to_history(event, i)
@@ -200,4 +206,4 @@ class MetadataBase(object):
 
 __all__ = ['MetadataBase']
 
-# vim:ts=4 sw=4 et
+# vim:ts=4 sts=4 sw=4 et
